@@ -32,6 +32,8 @@ import os
 import math
 import socket
 import mmap
+import gzip
+import codecs
 
 from . import const
 from .util import ip2long
@@ -78,11 +80,13 @@ class GeoIP(GeoIPBase):
         """
         Initialize the class.
 
-        @param filename: path to a geoip database
+        @param filename: path to a geoip database. If MEMORY_CACHE is used,
+            the file can be gzipped.
         @type filename: str
         @param flags: flags that affect how the database is processed.
-            Currently the only supported flags are STANDARD, MEMORY_CACHE, and
-            MMAP_CACHE.
+            Currently the only supported flags are STANDARD (the default),
+            MEMORY_CACHE (preload the whole file into memory), and
+            MMAP_CACHE (access the file via mmap).
         @type flags: int
         """
         self._filename = filename
@@ -92,11 +96,12 @@ class GeoIP(GeoIPBase):
             with open(filename, 'rb') as f:
                 self._filehandle = mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ)
 
-        else:
-            self._filehandle = open(filename, 'rb')
+        elif self._flags & const.MEMORY_CACHE:
+            self._filehandle = gzip.open(filename, 'rb')
+            self._memoryBuffer = self._filehandle.read()
 
-            if self._flags & const.MEMORY_CACHE:
-                self._memoryBuffer = self._filehandle.read()
+        else:
+            self._filehandle = codecs.open(filename, 'rb','latin_1')
 
         self._setup_segments()
 
@@ -114,7 +119,7 @@ class GeoIP(GeoIPBase):
         for i in range(const.STRUCTURE_INFO_MAX_SIZE):
             delim = self._filehandle.read(3)
 
-            if delim == six.b(chr(255) * 3):
+            if delim == six.u(chr(255) * 3):
                 self._databaseType = ord(self._filehandle.read(1))
 
                 if (self._databaseType >= 106):
@@ -136,10 +141,7 @@ class GeoIP(GeoIPBase):
                     buf = self._filehandle.read(const.SEGMENT_RECORD_LENGTH)
 
                     for j in range(const.SEGMENT_RECORD_LENGTH):
-                        if six.PY3:
-                            self._databaseSegments += (buf[j] << (j * 8))
-                        else:
-                            self._databaseSegments += (ord(buf[j]) << (j * 8))
+                        self._databaseSegments += (ord(buf[j]) << (j * 8))
 
                     if self._databaseType in (const.ORG_EDITION, const.ISP_EDITION):
                         self._recordLength = const.ORG_RECORD_LENGTH
@@ -205,10 +207,7 @@ class GeoIP(GeoIPBase):
 
             for i in range(2):
                 for j in range(self._recordLength):
-                    if six.PY3:
-                        x[i] += buf[self._recordLength * i + j] << (j * 8)
-                    else:
-                        x[i] += ord(buf[self._recordLength * i + j]) << (j * 8)
+                    x[i] += ord(buf[self._recordLength * i + j]) << (j * 8)
 
             if ipnum & (1 << depth):
 
@@ -245,8 +244,6 @@ class GeoIP(GeoIPBase):
         self._filehandle.seek(record_pointer, os.SEEK_SET)
 
         org_buf = self._filehandle.read(const.MAX_ORG_RECORD_LENGTH)
-        if six.PY3:
-            org_buf = org_buf.decode()
 
         return org_buf[:org_buf.index(chr(0))]
 
@@ -285,8 +282,9 @@ class GeoIP(GeoIPBase):
                 region = ''.join([chr((seek_region - const.CANADA_OFFSET) // 26 + 65), chr((seek_region - const.CANADA_OFFSET) % 26 + 65)])
             else:
                 i = (seek_region - const.WORLD_OFFSET) // const.FIPS_RANGE
-                if i in const.COUNTRY_CODES:
-                    country_code = const.COUNTRY_CODES[(seek_region - const.WORLD_OFFSET) // const.FIPS_RANGE]
+                if i < len(const.COUNTRY_CODES):
+                    #country_code = const.COUNTRY_CODES[(seek_region - const.WORLD_OFFSET) // const.FIPS_RANGE]
+                    country_code = const.COUNTRY_CODES[i]
                 else:
                     country_code = ''
                 region = ''
@@ -321,7 +319,8 @@ class GeoIP(GeoIPBase):
         record = {}
 
         record_buf_pos = 0
-        char = record_buf[record_buf_pos] if six.PY3 else ord(record_buf[record_buf_pos])
+        char = ord(record_buf[record_buf_pos])
+        #char = record_buf[record_buf_pos] if six.PY3 else ord(record_buf[record_buf_pos])
         record['country_code'] = const.COUNTRY_CODES[char]
         record['country_code3'] = const.COUNTRY_CODES3[char]
         record['country_name'] = const.COUNTRY_NAMES[char]
@@ -329,10 +328,10 @@ class GeoIP(GeoIPBase):
         str_length = 0
 
         # get region
-        char = record_buf[record_buf_pos+str_length] if six.PY3 else ord(record_buf[record_buf_pos+str_length])
+        char = ord(record_buf[record_buf_pos+str_length])
         while (char != 0):
             str_length += 1
-            char = record_buf[record_buf_pos+str_length] if six.PY3 else ord(record_buf[record_buf_pos+str_length])
+            char = ord(record_buf[record_buf_pos+str_length])
 
         if str_length > 0:
             record['region_name'] = record_buf[record_buf_pos:record_buf_pos+str_length]
@@ -341,10 +340,10 @@ class GeoIP(GeoIPBase):
         str_length = 0
 
         # get city
-        char = record_buf[record_buf_pos+str_length] if six.PY3 else ord(record_buf[record_buf_pos+str_length])
+        char = ord(record_buf[record_buf_pos+str_length])
         while (char != 0):
             str_length += 1
-            char = record_buf[record_buf_pos+str_length] if six.PY3 else ord(record_buf[record_buf_pos+str_length])
+            char = ord(record_buf[record_buf_pos+str_length])
 
         if str_length > 0:
             record['city'] = record_buf[record_buf_pos:record_buf_pos+str_length]
@@ -355,10 +354,10 @@ class GeoIP(GeoIPBase):
         str_length = 0
 
         # get the postal code
-        char = record_buf[record_buf_pos+str_length] if six.PY3 else ord(record_buf[record_buf_pos+str_length])
+        char = ord(record_buf[record_buf_pos+str_length])
         while (char != 0):
             str_length += 1
-            char = record_buf[record_buf_pos+str_length] if six.PY3 else ord(record_buf[record_buf_pos+str_length])
+            char = ord(record_buf[record_buf_pos+str_length])
 
         if str_length > 0:
             record['postal_code'] = record_buf[record_buf_pos:record_buf_pos+str_length]
@@ -371,14 +370,14 @@ class GeoIP(GeoIPBase):
         latitude = 0
         longitude = 0
         for j in range(3):
-            char = record_buf[record_buf_pos] if six.PY3 else ord(record_buf[record_buf_pos])
+            char = ord(record_buf[record_buf_pos])
             record_buf_pos += 1
             latitude += (char << (j * 8))
 
         record['latitude'] = (latitude/10000.0) - 180.0
 
         for j in range(3):
-            char = record_buf[record_buf_pos] if six.PY3 else ord(record_buf[record_buf_pos])
+            char = ord(record_buf[record_buf_pos])
             record_buf_pos += 1
             longitude += (char << (j * 8))
 
@@ -388,7 +387,7 @@ class GeoIP(GeoIPBase):
             dmaarea_combo = 0
             if record['country_code'] == 'US':
                 for j in range(3):
-                    char = record_buf[record_buf_pos] if six.PY3 else ord(record_buf[record_buf_pos])
+                    char = ord(record_buf[record_buf_pos])
                     record_buf_pos += 1
                     dmaarea_combo += (char << (j*8))
 
